@@ -1364,16 +1364,18 @@ rsvg_entity_decl (void *data, const xmlChar *name, int type,
   xmlEntityPtr entity;
   char *dupname;
 
-  entity = g_new (xmlEntity, 1);
+  entity = g_new0 (xmlEntity, 1);
   entity->type = type;
   entity->len = strlen (name);
   dupname = g_strdup (name);
   entity->name = dupname;
   entity->ExternalID = g_strdup (publicId);
   entity->SystemID = g_strdup (systemId);
-  entity->content = xmlMemStrdup (content);
-  entity->length = strlen (content);
-  entity->orig = NULL;
+  if (content)
+    {
+      entity->content = xmlMemStrdup (content);
+      entity->length = strlen (content);
+    }
   g_hash_table_insert (entities, dupname, entity);
 }
 
